#include <sys/socket.h> /* for accept() */
#include <arpa/inet.h>  /* for sockaddr_in and inet_ntoa() */
#include <stdio.h>  /* for perror() */
#include <stdlib.h> /* for exit() */
#include <string.h>     /* for memset() */
#include <unistd.h>
#include <sys/poll.h>
#include <sys/time.h>
#include <time.h>

void DieWithError(char *errorMessage)
{
    perror(errorMessage);
    exit(1);
}


int AcceptTCPConnection(int servSock)
{
    int clntSock;                    /* Socket descriptor for client */
    struct sockaddr_in echoClntAddr; /* Client address */
    unsigned int clntLen;            /* Length of client address data structure */

    /* Set the size of the in-out parameter */
    clntLen = sizeof(echoClntAddr);
    
    /* Wait for a client to connect */
    if ((clntSock = accept(servSock, (struct sockaddr *) &echoClntAddr, 
           &clntLen)) < 0)
        DieWithError("accept() failed");
    
    /* clntSock is connected to a client! */
    
    //printf("Handling client %s\n", inet_ntoa(echoClntAddr.sin_addr));

    return clntSock;
}


//////////////////////////////////////////////////

#define MAXPENDING 5    /* Maximum outstanding connection requests */


int CreateTCPServerSocket(unsigned short port)
{
    int sock;                        /* socket to create */
    struct sockaddr_in echoServAddr; /* Local address */


    /* Create socket for incoming connections */
    if ((sock = socket(PF_INET, SOCK_STREAM, IPPROTO_TCP)) < 0)
        DieWithError("socket() failed");
      
    /* Construct local address structure */
    memset(&echoServAddr, 0, sizeof(echoServAddr));   /* Zero out structure */
    echoServAddr.sin_family = AF_INET;                /* Internet address family */
    echoServAddr.sin_addr.s_addr = htonl(INADDR_ANY); /* Any incoming interface */
    echoServAddr.sin_port = htons(port);              /* Local port */

	int reuse = 1;
	setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&reuse, sizeof(reuse));

    /* Bind to the local address */
    if (bind(sock, (struct sockaddr *) &echoServAddr, sizeof(echoServAddr)) < 0)
        DieWithError("bind() failed");

    /* Mark the socket so it will listen for incoming connections */
    if (listen(sock, MAXPENDING) < 0)
        DieWithError("listen() failed");

    return sock;
}

int CreateUDPServerSocket(unsigned short port)
{
    int sock;                        /* socket to create */
    struct sockaddr_in echoServAddr; /* Local address */

    /* Create socket for incoming connections */
    if ((sock = socket(AF_INET, SOCK_DGRAM, 0)) < 0)
        DieWithError("socket() failed");
      
    /* Construct local address structure */
    memset(&echoServAddr, 0, sizeof(echoServAddr));   /* Zero out structure */
    echoServAddr.sin_family = AF_INET;                /* Internet address family */
    echoServAddr.sin_addr.s_addr = htonl(INADDR_ANY); /* Any incoming interface */
    echoServAddr.sin_port = htons(port);              /* Local port */

    /* Bind to the local address */
    if (bind(sock, (struct sockaddr *) &echoServAddr, sizeof(echoServAddr)) < 0)
        DieWithError("bind() failed");

    return sock;
}

static int instance_detection_socket=0;

int check_running_instance(int port)
{
	int reuse, tries;
	struct sockaddr_in listener_endpoint;
	
	instance_detection_socket = socket(PF_INET, SOCK_STREAM, 0);
	listener_endpoint.sin_family = AF_INET;
	listener_endpoint.sin_port = htons(port);
	listener_endpoint.sin_addr.s_addr = htonl(INADDR_ANY);
	setsockopt(instance_detection_socket, SOL_SOCKET, SO_REUSEADDR, &reuse, sizeof(reuse) );
	
	tries = 0;
	while( bind(instance_detection_socket, (struct sockaddr *)&listener_endpoint, sizeof(listener_endpoint)) < 0 )
	{
		tries++;
		if( tries == 10 ) return 1;
		usleep(100000);
	}
	if (listen(instance_detection_socket, MAXPENDING) < 0) return 1;
	return 0;
}

int network_write(int fd, unsigned char *buffer, unsigned int length, int override_timeout)
{
	//write with time-out
	struct pollfd pfd;
	int write_result;
	
	pfd.fd = fd;
	pfd.events = POLLOUT | POLLERR;
	pfd.revents = 0;
	
	if(poll(&pfd, 1, override_timeout) != 1) return -2;
	
	if(pfd.revents & POLLOUT)
	{
		write_result = write(fd, buffer, length);
		//debug_output(buffer, write_result, "e> ");
		return write_result;
	}
	else
		return -1;
}

int network_read(int fd, unsigned char *buffer, unsigned int length, int override_timeout)
{
	//read with time-out
	struct pollfd pfd;
	int read_result;
	
	pfd.fd = fd;
	pfd.events = POLLIN | POLLERR;
	pfd.revents = 0;
	
	//if( override_timeout < 0 ) override_timeout = timeout;
	
	if(poll(&pfd, 1, override_timeout) != 1) return -2;
	
	if(pfd.revents & POLLIN)
	{
		read_result = read(fd, buffer, length);
		//debug_output(buffer, read_result, "e< ");
		return read_result;
	}
	else
		return -1;
}

unsigned int GetTickCount(struct timeval initime)
{
    struct timeval tv;
    gettimeofday( &tv, NULL );
    // this will rollover ~ every 49.7 days
    return (unsigned int)( (tv.tv_sec-initime.tv_sec) * 1000 + tv.tv_usec / 1000);
}

int __nsleep(const struct timespec *req, struct timespec *rem)
{
    struct timespec temp_rem;
    if(nanosleep(req,rem)==-1)
        __nsleep(rem,&temp_rem);
    return 1;
}

int msleep(unsigned long milisec)
{
    struct timespec req={0},rem={0};
    time_t sec=(int)(milisec/1000);
    milisec=milisec-(sec*1000);
    req.tv_sec=sec;
    req.tv_nsec=milisec*1000000L;
    __nsleep(&req,&rem);
    return 1;
}

